pro adrlb,iter,psf,data,decon,unc,dark,mode,n,t,div,absthr,chithr,$
	likenew,imax,jmax,imagp,ww,wthr,afac,dimpsf,dime,w,calc,b,SILENT=silent
;+
;NAME
;     	adrlb
;
;PURPOSE
;
;	 ANDRIL ENGINE
;
;CALLING SEQUENCE:
;   	adrlb,iter,psf,data,decon,unc,dark,mode,n,t,div,absthr,chithr,$
;	likenew,imax,jmax,imagp,ww,thr,afac,dimpsf,dime,w,calc,b
;
;	adrlb,iter,psf,data,decon,unc,dark,mode,n,t,div,absthr,chithr,$
;	likenew,imax,jmax,imagp,ww,thr,afac,dimpsf,dime,w,calc,b,/SILENT
;
;
;INPUTS:
;	B 			is the PEDESTAL Factor
;	Caution 	some of the inputs are redundant in order to speed-up
;				calculations
;	iter		is the present iteration step varied in the outer loop
;	psf 		is the externally supplied PSF
;	data 		is the supplied image to be deconvolved (in Photons - Poisonian)
;	unc 		is the supplied uncertainty array (in Photons)
;	decon 		is the first approximation/result of the iteration (usually flat)
;	mode		is the mode (0 for classical, 1 for accelerated)
;	N, T  		are the parameters regulating the U - smoothing  (3,0.2)
;	div			is the superresolution factor (usually 5)
;	absthr		is the average absolute deviation for pixeles above threshold
;	chithr		is the normalised chi^2 value for pixels above threshold
;	likenew		is the natural Log of likelihood
;	imax,jmax	are externally supplied maximum dimensions for the weight corrections
;	ratp		is the externally supplied array of ratios observed/calculated signal
;
;	ww			is the correction factor array used to improve old model (decon)
;	wthr			is the externally supplied vector of pixels selected above threshold
;	afac		is the external optional factor allowing to scale the calculation aplitude (set to 1)
;	dimpsf		is the dimension of the PSF expressed in SXT pixels
;	dime		is the dimension of square data array
;	w			is the array of weights calculated externally by diwalim1
;   calc		is the calculated signal in pixels
;
; OPTIONAL INPUT KEYWORDS:
;SILENT   - 	doesn't show the windows  for the plots: U-Function, Correction matrix, abs(obs-cal)/unc,
;					ratio observed/calculated and deconvolved image
;
; METHOD:
;
;	Calculates only a single iteration so outer loop necessary
;	Image restoration Using the Damped
; 	Richardson-Lucy Method, in The restoration of HST Image and Spectra II,
;	Space Telecsope Science institute, 1994
;	R.J. Hanisch and R.L.White eds, 104-110
;	gives possibility of classic or accelerated calculations
;	uses inputs expressed in photons
;	uses Varosi convolution procedure
;
;HISTORY:
;	written, Janusz Sylwester, Space Research Centre, Polish Acad.
;	of Sciences 1996 - 1999
;	Oct 98, inserted option for b - pedestal factor
;-

;	Set-up the correcting ratio array. The size is the size of data incresaed
;	by the belt of the width equal to half of the PSF
		ratp=imagp*0.+ 1.d0		;fixes the ratio to one - also on the extended edges

;	setting the offset between extended and real arrays
		sdim=dimpsf/2+1
;	and fills the inside of the extended array with the data
		imagp(sdim:sdim+dime-1,sdim:sdim+dime-1)=data

;
;	Set-up the correcting ratio array. The size is the size of data incresaed
;	by the belt of the width equal to half of the PSF
;		ratp=imagp*0.+ 1.d0		;fixes the ratio to one - also on the extended edges

;		ratp=smooth(ratp,sdim)
;	convolve the image using Varosi convolution programme
        con = convolve(decon,psf, FT_PSF = psf_FT)
        con = positivi(con,epsilon=1.e-30)

;   calculating the expected signal in pixels
        calc=calctst(psf,dime,div,con)

;	setting the likelihood
		likenew=nlikely(data,calc+dark+b)		;adds dark current to calculations
											; L according to White, 1994 Eq.(2)
;   calculating U -function as in White, 1994
	 	ratp(sdim:sdim+dime-1,sdim:sdim+dime-1)=ufun(n,t,(data)/afac/unc,(calc+dark+b)/afac/unc)
 if not keyword_set(silent) then begin
	 	wset,2
		plot,(data)/((calc+dark+b)),ufun(n,t,data,(calc+dark+b)),psym=2
		xyouts,10,10,string(format='(I4)',iter),/device,size=2

	 	wset,5
	 	plot,((data)/unc),((calc+dark+b)/unc),psym=2,xst=1,yst=1,$
	 		xra=[0,max([max((data)/unc),max((calc+dark+b)/unc)])],$
	 		yra=[0,max([max((data)/unc),max((calc+dark+b)/unc)])]
	 	oplot,[0,max([max((data)/unc),max((calc+dark+b)/unc)])],$
	 		  [0,max([max((data)/unc),max((calc+dark+b)/unc)])]
		xyouts,10,10,string(format='(I4)',iter),/device,size=2
		wait,1
endif
		;in the above line we add the dark current etc correction to
		;compare with the uncorrected data (always positive)

;	setting correction for the accelerated mode
		if (mode ge 1 and iter gt 2) then ratp=ratp-1.d0

;     	Calculates
;		average absolute difference and chi^2 in terms of uncertainty
;		for pixels above threshold
		if (wthr(0) gt -1) then begin
			absthr=total((abs((data(wthr))/afac-(calc(wthr)+dark(wthr)+b)/afac))/(unc(wthr)/afac))/n_elements(wthr)
		;calculates chisq for points above threshold thr
        	chithr=chisq((data(wthr))/afac,(calc(wthr)+dark(wthr)+b)/afac,unc(wthr)/afac)
        endif else begin  ; for all points
			absthr=total((abs((data)/afac-(calc+dark+b)/afac))/(unc/afac))/n_elements(calc)
		;calculates chisq for points above threshold thr
        	chithr=chisq((data)/afac,(calc+dark+b)/afac,unc/afac)
		endelse

;		starting main weight factor calculation loop
       	for j=0,imax do begin
       		for i=0,jmax do begin
			ip=i/div		;start index
			ik=ip + dimpsf+1	;end index
							;+1

			jp=j/div        	;start index
			jk=jp + dimpsf+1        ;end index

           		;calculating correction array
           		ww(i,j)=total(w(i mod div,j mod div,*,*)*ratp(ip:ik,jp:jk))
       		endfor
        endfor
;**************
;		ww=shift(ww,1,-1)
;		correcting new model (classical R-L)
		deconn=decon*ww

;		accelerated R-L case
;		starting on August 2nd, 98 uses chi^2 optimization for alpha
		if (mode ge 1 and iter gt 2) then begin
				enti=dblarr(3) 	;values of likelihood three numbers for parabola
               	newmin=enti		;values of minimum for new model
		;	finding the maximum of max likelihood
               	for ii=0,2 do begin
					alpha=ii*1.d0
					deconn=decon*(1.+ alpha*ww)
					newmin(ii)=min(deconn)  ;setting the minimum
					;	convolve the image to calculate the fluxes
					con=convolve(deconn,psf,FT_PSF = psf_FT)
					con=positivi(con,epsilon=1.e-30)
					;   calculating the expected rates in pixels
				   	calr=calctst(psf,dime,div,con) 	;in Photons this gives array
				   											;of calculated intensities
					;	calculating the likelihood
				    ; 	uses the reversed chi^2 condition instead of errornous likelihood expression
				    ; 	tries to optimize (maximally decrease chi^2)
				    ;enti(ii)=1./chisq((data)/afac,(calr+dark+b)/afac,unc/afac)	;August 2nd, 98
				    enti(ii)=-1./nlikely(data,calr+dark+b)	;August 2nd, 98
		  		endfor
		;	calculating the position of maximum
				parabola,findgen(3),enti,r,mm,zeros
				alpha=mm(0)        ;setting alpha to Xmax
				if alpha le 0. then begin
				alpha=1.
				goto, pipi
				endif
				;	calculating values for the maximum
				deconn=decon*(1.+ alpha*ww)
				newm=min(deconn)  ;setting the minimum
				;	checking whether new model for maximum is valid to use
				;	if not then
		 		newma=0.
		; 		print,iter,alpha
				if newm lt 1e-30 then begin
				;	searching for the positive crossing at the level of 1d-30
				;    if max((zeros)) lt 0 then alpha=max(abs(zeros)) else  alpha=max(zeros)	;takes the larger crossing point

				    					;prevents from
				alpha=max(zeros)	;takes the larger crossing point or fixed value of 3
;		print,alpha
					pipi:
						deconn=decon*(1.+ alpha*ww)
						newma=min(deconn)  ;setting the minimum
						;checks whether it is positive
						if newma lt 1e-30 then begin
						;if not decreases alpha to get it above 1e-30
							alpha=0.7*alpha
							goto,pipi
						endif
				endif
;				print,zeros,alpha
;	 		if iter ge 3 then stop

			endif   ;accelerated loop
;	rewriting the new approximation over old  above the level epsilon
		decon=deconn
;	convolve the image using Varosi convolution programme
        con = convolve(decon,psf, FT_PSF = psf_FT)
;	makes sure the result is positive 1-August-98
        con = positivi(con,epsilon=1.e-30)

;   calculating the final signal in pixels
        calc=calctst(psf,dime,div,con)

end
