pro DRLB,niter,mode,thr,image,uncer,dark,dime, div,n,t,psf,dimpsf,b,decon,edge,SILENT=silent

;+
;NAME
;	DRLB  Version 3.0
;	includes pedestal b
;
;PURPOSE
;	GENERAL ANDRIL ROUTINE FOR USER SPECIFIED IMAGE UNCERtainty,
;	PSF etc
;	sets-up deconvolution of  PSF from  images
;
;CALLING SEQUENCE:
;   	DRLB,niter,mode,image,uncer,dark,dime, div,n,t,dimpsf,b,decon
;	DRLB,niter,mode,image,uncer,dark,dime, div,n,t,dimpsf,b,decon,/SILENT
;
;INPUTS:
;	niter	- is the termination step for the iteration
;	mode	- is the mode (0 for classical, 1 for accelerated)
;	image 	- is the image to be deconvolved
;	uncer 	- is the uncertainty provided from outside
;	dark 	- is the dark current
;	dime 	- is dimension of the image to be deconvolved
;	div	- is the superresolution factor (usually 5)
;	n,t  	- are the parameters regulating the U - smoothing  (3,0.2)
;	dimpsf	- is the size of the PSF expressed in SXT pixels
;	b	- is background pedestal level
;	decon 	- is result of deconvolution output
;
;OPTIONAL INPUT KEYWORDS:
;SILENT   - 	doesn't show the windows  for the plots: U-Function, correction matrix, abs(obs-cal)/unc,
;					ratio observed/calculated and deconvolved image
;
;METHOD:
;
;	Main routine for SXT deconvolution - sets-up calculations
;	Image restoration Using the Damped
; 	Richardson-Lucy Method, in The restoration of HST Image and Spectra II,
;	Space Telecsope Science institute, 1994
;	R.J. Hanisch and R.L.White eds, 104-110
;	gives possibility of classic or accelerated calculations
;	uses LINEAR inputs expressed in photons
;	uses Varosi convolution procedure
;
;
;HISTORY:
;	written, Janusz Sylwester, Space Research Centre, Polish Acad.
;	of Sciences February 	1998
;	modified to include b	1999
;-

        div=fix(div)		;make sure integer
        if niter eq 0 then decon=dblarr(dime*div,dime*div) ;initialize deconvolved array to zero
 		data=image			;set-up observation array
;	Checks-up for zeros present in the data array
		if min(data) le 0. then begin
			print,'There are data points LE 0'
			print,'Making array positive using Varosi POSITIVI'
			print,'with epsilon= 1.E-5'
			data=positivi(image,epsilon=1.d-5)
		endif
	    tc=double(total(data)-total(dark))    ;calculates total photon signal above dark


    	ave=tc/(double(dime*div))^2  		; average level for zero flatt approximation

		decon=decon+ave
	;	decon=dblarr(dime*div,dime*div)+ave ; sets up zero approximation array

	;	selects pixels above threshold WTHR IS SELECTION VECTOR
 		WThr=where(data/uncer ge thr)  	;select points with the accuracy better than 0.2



	;	Setting the dimensions of the working square array
	;	with the subpixel subresolution
        imax=div*dime-1
        jmax=div*dime-1
        imagp=dblarr(dime+dimpsf+1,dime+dimpsf+1) 	;sets up the data + boundary
													;convolution field

	;	fills the extended whole (data+boundary) region with the average signal from the
	;	actual image edges averaged over the image
		imagp=imagp+(total(data(0,*))+ total(data(dime-1,*))+$
					total(data(*,0)) + total(data(*,dime-1)))$
					/(4*n_elements(data(0,*)))

		ratp=dblarr(dime+dimpsf+1,dime+dimpsf+1)+1.d0 	;sets up the ratio array
														;convolution area: data + extended boundary
	;	Sets up the iteration weights array	ww
		ww=decon

	;	Set up presentation windows
	if not keyword_set(silent) then begin
		window,0,title='Deconvolved Image',xsize=300,ysize=300,xpos=0,ypos=20
		window,1,title='Correction matrix ',xsize=300,ysize=300,xpos=350,ypos=20
		window,2,title='U-Function N='+strcompress(string(n))+'  T='+string(format='(f3.1)',t),xsize=300,ysize=300,xpos=700,ypos=20
		window,3,title='Absolute (obs-cal)/unc',xsize=300,ysize=300,xpos=0,ypos=350
	   	window,5,title='Ratio observed/calculated',xsize=300,ysize=300,xpos=700,ypos=350
	endif
	;	Initialization of the log file andril.out for convergence results

		openw,1,'andril.out'  ;open file for output
		printf,1,format='(4a,2(a,i3),2(a,f5.2))','Andril run on '+ systime(),$
			' N = ',n,' T = ',t
		printf,1,format='((a,i3))','VERSION: 3.1  Mode:',mode
		printf,1,'iter       absdevth            chithr          likeli'

	;	calling the weight array calculation programme DIVALIM
		diwalim1,psf,dimpsf,div,w

	;	INITIALIZE VALUE of convergence characteristics and others
		likeold=-1.d30
		absth=dblarr(niter+1)
		chith=absth
		likel=absth

	;	Starts the main deconvolution loop from i=0 to niter (threshold step)
			psr=rotate(psf,2)				; inverting the psf in order to be used in adrl
			for i=0,niter do begin
				adrlb,i,psr,data,decon,uncer,dark,mode,n,t,div,absthr,chithr,$
						likenew,imax,jmax,imagp,ww,Wthr,1.,dimpsf,dime,w,calc,b,SILENT=silent
				absth(i)=absthr
				chith(i)=chithr
				likel(i)=likenew

				if not keyword_set(silent) then begin
					wset,0
					image_c,alog10(decon)   ;shows present model
					xyouts,10,10,string(format='(I4)',i),/device,size=2

					wset,1
					shade_surf,ww   ;correction surface
					xyouts,10,10,string(format='(I4)',i),/device,size=2

					wset,3
					image_c,(data-calc)/uncer   ;differences in sigma
					xyouts,10,10,string(format='(I4)',i),/device,size=2
        			endif
				wait,0.1


				;adding present results to the logfile
				printf,1,format='(i3,5(e17.8))',i,absthr,chithr,likenew
				; stop iterations if likelihood decreases
				if (likenew-likeold) lt 1.e-20 then goto, koniec

				likeold=likenew
			endfor
koniec:
			close,1  ; closing the log file

    ;	iter=i-1      ; setting the final iteration step

	;	Normalizing the results
	;	Normalization is important while b is not zero
	;	For b=0 the method conserves the flux by iself!

		new=decon/total(decon)*tc  ; result model normalized to total photons above dark

	;	Interpolates the original image to the applied superresolution for sake of comparison (with dark)

			; Saving the results
			bb=b(0,0) 	;save the pedestal level
			info=strarr(32)	; sets up the info

			info(0)=' These are results of ANDRIL code developed by J. & B. Sylwester                             '
			info(1)=' If publishing the results, please mention two references:                                   '
			info(3)=' ANDRIL - Maximum Likelihood Algorithm for Deconvolution of SXT Images                       '
			info(4)=' J. S y l w e s t e r   and   B. S y l w e s t e r   (Paper I)                               '
			info(5)=' ACTA ASTRONOMICA                                                                            '
			info(6)=' Vol. 48, (1998), pp. 519-545                                                                '
			info(7)=' and                                                                                         '
			info(8)=' Reconstruction of Images with Poisson Noise                                                 '
			info(9)=' J. S y l w e s t e r   and   B. S y l w e s t e r    (Paper II)                             '
			info(10)=' ACTA ASTRONOMICA                                                                            '
			info(11)=' (1999) - submitted                                                                         '
			info(12)=' VARIABLE DESCRIPTION in this save set                                                                                             '
			info(13)=' All images are in PHOTONS                                                                  '
			info(14)='                                                       '
			info(15)=' EDGE				 0 if no edges added in calculations, 1 if extra edge                                                                                           '
			info(17)=' BB                value of "background pedestal" (see Paper II)                            '
			info(18)=' CHITH             vector of  normalize chi^2 convergence (pixels with important signal S/N>5), if no point above threshold then the value is taken for all pixels'
			info(19)=' DARK              array of the dark current used                                           '
			info(20)=' DIMPSF            size of the PSF square array used in deconvolution (input pixels)        '
			info(21)=' DIV               over-resolution factor (usually 5)                                       '
			info(22)=' IMAGE, uncer      input image, (uncertainty) for deconvolution (Photons)                   '
			info(23)=' ITER              iteration limit (or iteration step where convergence stops)              '
			info(24)=' LIKEL             vector of log L(ikelihood)  convergence                                  '
			info(25)=' N    	         scalar defining the smoothing properties (see Paper I)                   '
			info(26)=' NEW               THE RESULT of DECONVOLUTION (photons, total counts preserved)            '
			info(27)=' PSF               the PSF array used for deconvolution                                     '
			info(28)=' T                 scalar defining the smoothing properties (see Paper I)                   '
			info(29)=' THR               threshold level for S/N ratio (5 is for 20% accuracy)                    '
			info(30)=' mode              mode used in the calculations (0 - classical RL, 1 - accelerated)        '
			info(31)=' calc              matrix of calculated fluxes (photons)'

			if (edge eq 1) then begin
				sd=dimpsf/2+1
				new=	float(new(sd*div:(dime-sd)*div-1,sd*div:(dime-sd)*div-1))			;changes to single precision & removes artificial edges
				dark=	float(dark(sd:dime-sd-1,sd:dime-sd-1))
				image=  float(image(sd:dime-sd-1,sd:dime-sd-1))
				calc=	float(calc(sd:dime-sd-1,sd:dime-sd-1))
				uncer=	float(uncer(sd:dime-sd-1,sd:dime-sd-1))
			endif else begin
				new=float(new)
				dark=float(dark)
				image=float(image)
				calc=float(calc)
				uncer=float(uncer)
			endelse
				dimpsf=	fix(dimpsf)
				psf=	float(psf)			;changes to single precision
				whi=	where(likel ne 0)	;searching for non-zero results
				iter=	fix(whi(n_elements(whi)-1))
				chith=	float(chith(0:iter))		;shortening arrays
				likel=	float(likel(0:iter))
			;	writing save file for andril
    		save,filename='andril.sav',info,bb,new,iter,chith,image,likel,$
					dark,mode,n,t,thr,psf,dimpsf,div,new,calc,uncer,edge
	end
